﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

namespace JayJayWon
{
    public class CombiTagFilterUI
    {
        private GameObject combiTagFilterBoxGO;

        private int currentResourceType;
        private string currentAtomPresetResourceType;

        private GameObject combiLogicBoxGO;
        private GameObject combiLogicListContentGO;
        private GameObject combiLogicListTagSelectionContentGO;
        private GameObject combiLogicTagSelectionBoxGO;
        private UIDynamicTextField combiLogicTagFilterTextBox;
        private UIDynamicButton closeButtonUID;

        private BrowserFiltersUI browserFiltersUI;
        public BrowserUI browserUI;

        private UIDynamicButton closeButton;

        private HashSet<CombiTagFilterConditionRowUI> combiTagFilterConditionRows = new HashSet<CombiTagFilterConditionRowUI>();
        private Dictionary<CombiLogicCondition, CombiTagFilterConditionRowUI> clcToClcUI = new Dictionary<CombiLogicCondition, CombiTagFilterConditionRowUI>();
        private HashSet<GameObject> combiTagSelectionRows = new HashSet<GameObject>();
        public CombiLogicCondition selectedCLC { get; private set; } = null;
        private CombiTagFilterConditionRowUI selectedCLCUI { get
            {
                if (clcToClcUI.ContainsKey(selectedCLC)) return clcToClcUI[selectedCLC];
                return null;
            } }
        private GameObject addCombiRowGO;

        private JSONStorableString combiTagFilterJSS = new JSONStorableString("combiTagFilter", "");

        private CombiTag currentCombiTag;

        public CombiTagFilterUI(GameObject combiTagFilterBoxGO, BrowserUI browserUI)
        {
            this.combiTagFilterBoxGO = combiTagFilterBoxGO;
            browserFiltersUI = browserUI.browserFiltersUI;
            this.browserUI = browserUI;
        }

        public void RefreshUI(int resourceType, string atomPresetResourceType, CombiTag newCombiTag)
        {
            ClearUI();

            currentCombiTag = newCombiTag;
            currentResourceType = resourceType;
            currentAtomPresetResourceType= atomPresetResourceType;

            bool fullWidthDesktopModeActive = !Globals.isVREnabled && browserUI.currentDesktopDisplayMode == DesktopDisplayMode.fullWidth;

            combiLogicBoxGO = UI.CreateVScrollableContent(combiTagFilterBoxGO, new Color(0f,0f, 0.1f, 0.5f), AnchorPresets.topLeft, 300f, 250f, new Vector2(0f, -20f), 10f, 3f);          
            combiLogicListContentGO = combiLogicBoxGO.transform.Find("scrollableContentViewportGO/vScrollableContentGO").gameObject;            

            combiLogicTagSelectionBoxGO = UI.CreateVScrollableContent(combiTagFilterBoxGO, new Color(0f, 0f, 0f, 0.5f),AnchorPresets.topLeft, 140f, 230f, new Vector2(305f, -40f), 10f, 1f);
            combiLogicListTagSelectionContentGO = combiLogicTagSelectionBoxGO.transform.Find("scrollableContentViewportGO/vScrollableContentGO").gameObject;
            combiLogicTagFilterTextBox = UI.CreateTextBox(combiTagFilterBoxGO, combiTagFilterJSS, 140f, 20f, 22, 305f, -20f, AnchorPresets.topLeft, true, InputField.LineType.SingleLine, "Filter Tags list...", 0f, 0f, CombiTagFilterNameChangeCallback);

            RefreshCombiTagSelectionUI();

            var textBox = UI.CreateBasicTextBox(combiTagFilterBoxGO,"Select resources....", 400f, 20f, 20, 0f, 0f,AnchorPresets.topLeft,backgoundColor: new Color(0f, 0f, 0f, 0f));

            GameObject textGO = textBox.transform.Find("text").gameObject;
            textGO.GetComponent<Text>().color = Color.white;

            closeButtonUID = UI.CreateUIButton(combiTagFilterBoxGO, 25f, 25f, "^", 32, 0f, -25f, AnchorPresets.bottomRight);
            closeButtonUID.button.onClick.AddListener(delegate
            {
                browserFiltersUI.ToggleCombiTagUI();
            });
            closeButtonUID.buttonColor = Color.cyan;
            browserUI.ApplyHelp(closeButtonUID, "Hide the Tag Combination filter setup window.");

            RefreshCombiTagConditionsList(newCombiTag);
        }
        private void ClearUI()
        {
            if (combiLogicTagSelectionBoxGO != null) GameObject.Destroy(combiLogicTagSelectionBoxGO);
            if (combiLogicTagFilterTextBox != null) GameObject.Destroy(combiLogicTagFilterTextBox.gameObject);
            if(closeButtonUID!=null) GameObject.Destroy(closeButtonUID.gameObject);

            if (combiLogicBoxGO != null) GameObject.Destroy(combiLogicBoxGO);
            if (closeButton != null) GameObject.Destroy(closeButton.gameObject);
        }

        private void ClearCombiTagConditionsList()
        {
            foreach (var ctcr in combiTagFilterConditionRows)
            {
                ctcr.ClearUI();
            }
            combiTagFilterConditionRows.Clear();
            clcToClcUI.Clear();
            if (addCombiRowGO != null) GameObject.Destroy(addCombiRowGO);
        }

        private void RefreshCombiTagConditionsList(CombiTag combiTag)
        {
            ClearCombiTagConditionsList();
            var combiLogicStatements = combiTag.combiLogicConditions;
            int rowIndex = 1;
            foreach (CombiLogicCondition cl in combiLogicStatements)
            {
                var combiTagConditionsRow = new CombiTagFilterConditionRowUI(combiLogicListContentGO, cl, rowIndex, this);
                clcToClcUI.Add(cl, combiTagConditionsRow);
                combiTagFilterConditionRows.Add(combiTagConditionsRow);
                rowIndex++;
            }
            addCombiRowGO = UI.AddChildGOImage(combiLogicListContentGO, new Color(0f, 0f, 0f, 0f), AnchorPresets.hStretchTop, 300f, 20f, Vector2.zero);
            var addButton = UI.CreateUIButton(addCombiRowGO, 60f, 20f, "Add...", 22, -10f, 0f, AnchorPresets.middleRight);
            addButton.button.onClick.AddListener(delegate
            {
                combiTag.AddCombiLogicCondition();
                selectedCLC = null;
                RefreshUICurrentResourceType();
            });
            browserUI.ApplyHelp(addButton, "Add a new logic condition to the selection criteria");
        }

        private void RefreshUICurrentResourceType()
        {
            RefreshUI(currentResourceType, currentAtomPresetResourceType, currentCombiTag);
        }

        private void CombiTagFilterNameChangeCallback(string tagName)
        {
            string tagNameMod = TagManifest.NormalizeTagName(tagName);

            combiTagFilterJSS.inputField.text = tagNameMod;
            RefreshCombiTagSelectionUI();
        }

        public void SelectTagsActive(CombiTagFilterConditionRowUI rowUI)
        {
            if (selectedCLC != null && selectedCLCUI !=null && selectedCLCUI.tagNameListButton != null) selectedCLCUI.tagNameListButton.buttonColor = BrowserUI.lightGrey;
            selectedCLC = rowUI.clc;
            RefreshCombiTagSelectionUI();
        }

        public void RefreshCombiTagSelectionUI()
        {

            if (selectedCLC != null && selectedCLC.isTaggableCondition)
            {
                combiLogicTagFilterTextBox.gameObject.SetActive(true);
                combiLogicTagSelectionBoxGO.SetActive(true);
                combiLogicListTagSelectionContentGO.SetActive(true);

                foreach (GameObject go in combiTagSelectionRows)
                {
                    GameObject.Destroy(go);
                }
                combiTagSelectionRows.Clear();

                RefreshCombiTagSelectionList();
            }
            else
            {
                combiLogicTagFilterTextBox.gameObject.SetActive(false);
                combiLogicTagSelectionBoxGO.SetActive(false);
                combiLogicListTagSelectionContentGO.SetActive(false);
            }
        }

        private UIDynamicButton lastTagsButtonActivate = null;
        private void RefreshCombiTagSelectionList()
        {
            
            foreach (Tag tag in TagManifest.GetTags(currentResourceType, currentAtomPresetResourceType))
            {
                if ((tag.tagCategory != TagCategory.combi || !currentCombiTag.usedByCombiTagsHieararchy.Contains(tag as CombiTag)) && tag != currentCombiTag)
                {
                    
                    if (combiTagFilterJSS.inputField.text == "" || tag.tagName.Contains(combiTagFilterJSS.inputField.text))
                    {
                        var rowGO = UI.AddChildGOImage(combiLogicListTagSelectionContentGO, new Color(0, 0, 0, 0), AnchorPresets.hStretchTop, 130f, 20f, Vector2.zero);
                        combiTagSelectionRows.Add(rowGO);

                        bool selected = selectedCLC.usesTags.Contains(tag);
                        if (selectedCLC.logicConditionJSEnum.val == CombiLogicConditionOptions.taggedWith)
                        {
                            var button = UI.CreateUIButton(rowGO, 130f, 20f, tag.tagName, 22, anchorPreset: AnchorPresets.middleLeft);
                            browserUI.ApplyHelp(button, "Select this tag for the '" + selectedCLC.logicConditionJSEnum.displayVal + "' condition.");
                            if (tag.tagCategory == TagCategory.combi) button.buttonText.text = "<i>" + tag.tagName + "</i>";
                            if (selected)
                            {
                                button.buttonColor = Color.cyan;
                                lastTagsButtonActivate = button;
                            }
                            else button.buttonColor = BrowserUI.lightGrey;
                            button.button.onClick.AddListener(delegate
                            {
                                bool selected2 = selectedCLC.usesTags.Contains(tag);
                                if (!selected2)
                                {
                                    if (lastTagsButtonActivate != null) lastTagsButtonActivate.buttonColor = BrowserUI.lightGrey;
                                    selectedCLC.UseTag(tag, true);
                                    button.buttonColor = Color.cyan;
                                    lastTagsButtonActivate = button;
                                }
                                else
                                {
                                    selectedCLC.RemoveUseTag(tag);
                                    button.buttonColor = BrowserUI.lightGrey;
                                }
                                selectedCLCUI.UpdateTagListText();
                                browserUI.RefreshFilterResults();
                                browserUI.browserFiltersUI.RefreshCombiTagUI();
                            });
                        }
                        else
                        {
                            string tagName = tag.tagName;
                            if (tag.tagCategory == TagCategory.combi) tagName = "<i>" + tag.tagName + "</i>";

                            var toggle = UI.CreateToggle(rowGO, new JSONStorableBool("", selected), 130f, 20f, tagName, 22, anchorPreset: AnchorPresets.middleLeft, checkBoxSize: 20f);
                            if (selected) toggle.backgroundColor = Color.cyan;
                            else toggle.backgroundColor = BrowserUI.lightGrey;
                            browserUI.ApplyHelp(toggle, "Select this tag for the '" + selectedCLC.logicConditionJSEnum.displayVal + "' condition.");

                            toggle.toggle.onValueChanged.AddListener(delegate
                            {
                                bool selected2 = selectedCLC.usesTags.Contains(tag);

                                if (!selected2)
                                {
                                    selectedCLC.UseTag(tag);
                                    toggle.backgroundColor = Color.cyan;
                                }
                                else
                                {
                                    selectedCLC.RemoveUseTag(tag);
                                    toggle.backgroundColor = BrowserUI.lightGrey;
                                }
                                selectedCLCUI.UpdateTagListText();
                                browserUI.RefreshFilterResults();
                                browserUI.browserFiltersUI.RefreshCombiTagUI();
                            });
                        }

                    }

                }

            }
        }

        public void DeleteCombiTagConditionRow(CombiLogicCondition clc)
        {
            currentCombiTag.RemoveCombiLogicConditions(clc);
            RefreshUICurrentResourceType();
            browserUI.RefreshFilterResults();
            browserUI.browserFiltersUI.RefreshCombiTagUI();
        }

        public void RefreshCombiConditionsLogicOperand(CombiTagFilterConditionRowUI clcUIexception)
        {
            foreach (var rowUI in combiTagFilterConditionRows)
            {
                if (rowUI != clcUIexception)
                {
                    rowUI.RefreshUI();
                }
            }
        }

    }


    public class CombiTagFilterConditionRowUI
    {
        private GameObject rowGO;
        public CombiLogicCondition clc { get; private set; }
        private int rowIndex;

        private UIDynamicPopup tagOperandPopup;
        private UIDynamicPopup logicConditionPopup;
        public UIDynamicButton tagNameListButton { get; private set; }
        private UIDynamicButton rowDeleteButton;
        private GameObject thatAreTextBoxGO;

        private GameObject parentGO;

        private CombiTagFilterUI parentUI;

        public CombiTagFilterConditionRowUI(GameObject parentGO, CombiLogicCondition cls, int rowIndex, CombiTagFilterUI parentUI)
        {
            this.clc = cls;
            this.rowIndex = rowIndex;
            this.parentGO = parentGO;
            this.parentUI = parentUI;
            rowGO = UI.AddChildGOImage(parentGO, Color.grey, AnchorPresets.hStretchTop, 290f, 40f, Vector2.zero);

            CreateUI();
        }

        public void ClearUI()
        {
            if (rowGO != null) GameObject.Destroy(rowGO);
        }

        public void RefreshUI()
        {
            if (logicConditionPopup != null) GameObject.Destroy(logicConditionPopup.gameObject);
            if (tagNameListButton != null) GameObject.Destroy(tagNameListButton.gameObject);
            if (tagOperandPopup != null) GameObject.Destroy(tagOperandPopup.gameObject);
            if (thatAreTextBoxGO != null) GameObject.Destroy(thatAreTextBoxGO);
            if (rowDeleteButton != null) GameObject.Destroy(rowDeleteButton.gameObject);
            CreateUI();
        }

        private void CreateUI()
        {
            if (rowIndex == 1) thatAreTextBoxGO = UI.CreateBasicTextBox(rowGO, "that are", 50f, 20f, 22, 0f, 0f, AnchorPresets.topLeft, false, 18, HorizontalWrapMode.Overflow, null);
            else if (rowIndex == 2)
            {
                tagOperandPopup = UI.CreatePopup(rowGO, clc.parentCombiTag.logicOperandJSEnum, 50f, 20f, null, 22, 0f, 0f, AnchorPresets.topLeft, labelProportion: 0f, popupRowHeight: 20f);
                parentUI.browserUI.ApplyHelp(tagOperandPopup, "The logical operation that will combine multiple conditions for this tag combination filter.");
                tagOperandPopup.popup.onOpenPopupHandlers += () =>
                {
                    Globals.mvrScript.StartCoroutine(ReparentPopup(tagOperandPopup));
                };
                tagOperandPopup.popup.onValueChangeHandlers += (string val) =>
                {
                    parentUI.RefreshCombiConditionsLogicOperand(this);

                    parentUI.SelectTagsActive(this);
                    RefreshUI();
                    parentUI.browserUI.RefreshFilterResults();
                    parentUI.browserUI.browserFiltersUI.RefreshCombiTagUI();
                };
            }
            else thatAreTextBoxGO = UI.CreateBasicTextBox(rowGO, clc.parentCombiTag.logicOperandJSEnum.displayVal, 50f, 20f, 22, 0f, 0f, AnchorPresets.topLeft, false, 18, HorizontalWrapMode.Overflow, null, true, TextAnchor.MiddleCenter);

            logicConditionPopup = UI.CreateScrollablePopup(rowGO, clc.logicConditionJSEnum, 120f, 20f, "", 22, 60f, -2f, 250f, false, true, AnchorPresets.topLeft, 0f, 20f, 22, true, 20f);
            parentUI.browserUI.ApplyHelp(logicConditionPopup, "Select a condition that will determine whether resources are selected based on their tags");


            logicConditionPopup.popup.onOpenPopupHandlers += () =>
            {
                Globals.mvrScript.StartCoroutine(ReparentPopup(logicConditionPopup));
            };

            if (rowIndex > 1)
            {
                rowDeleteButton = UI.CreateUIButton(rowGO, 20f, 20f, "X", 20, 10f, 0f, AnchorPresets.bottomLeft);
                parentUI.browserUI.ApplyHelp(rowDeleteButton, "Remove this condition from the list of conditions applied to this tag filter selection");
                rowDeleteButton.buttonColor = Color.red;
                rowDeleteButton.textColor = Color.white;
                rowDeleteButton.button.onClick.AddListener(delegate
                {
                    parentUI.DeleteCombiTagConditionRow(clc);
                });
            }

            logicConditionPopup.popup.onValueChangeHandlers += (string val) =>
            {
                if (clc.logicConditionJSEnum.val == CombiLogicConditionOptions.taggedWith) clc.UseOnlyOneTag();
                parentUI.SelectTagsActive(this);
                parentUI.RefreshCombiTagSelectionUI();

                RefreshUI();
                parentUI.browserUI.RefreshFilterResults();
                parentUI.browserUI.browserFiltersUI.RefreshCombiTagUI();
            };
            if (clc.isTaggableCondition)
            {
                tagNameListButton = UI.CreateUIButton(rowGO, 100f, 40f, "", 20, 0f, 0, AnchorPresets.topRight, true);
                parentUI.browserUI.ApplyHelp(tagNameListButton, "The list of tags that this condition is assessed against. Activate the button and select tags in the right hand list to modify.");
                if (parentUI.selectedCLC == clc)
                {
                    tagNameListButton.buttonColor = Color.cyan;
                }
                else
                {
                    tagNameListButton.buttonColor = BrowserUI.lightGrey;
                }


                tagNameListButton.button.onClick.AddListener(delegate
                {
                    parentUI.SelectTagsActive(this);
                    tagNameListButton.buttonColor = Color.cyan;
                    UpdateTagListText();
                });

                UpdateTagListText();
            }

        }

        public void UpdateTagListText()
        {
            if (clc.usesTags.Count == 0)
            {
                tagNameListButton.buttonText.alignment = TextAnchor.MiddleCenter;
                tagNameListButton.buttonText.text = "Select tags...";
                tagNameListButton.buttonText.fontSize = 22;
            }
            else
            {
                tagNameListButton.buttonText.alignment = TextAnchor.UpperLeft;
                tagNameListButton.buttonText.text = clc.GetTagNameList();
                tagNameListButton.buttonText.fontSize = 20;
            }
        }

        protected IEnumerator ReparentPopup(UIDynamicPopup popupUID)
        {
            yield return new WaitForEndOfFrame();

            if (popupUID.popup.visible)
            {
                popupUID.popup.popupPanel.transform.SetParent(parentGO.transform.parent.parent.parent.parent);
            }
        }

    }

}
